@@ -404,6 +404,20 @@ class Factory
         // add installers to the manager (must happen after download manager is created since they read it out of $composer)
         $this->createDefaultInstallers($im, $composer, $io, $process);
 
+        // init locker if possible
+        if ($composer instanceof Composer && isset($composerFile)) {
+            $lockFile = self::getLockFile($composerFile);
+            if (!$config->get('lock') && file_exists($lockFile)) {
+                $io->writeError('<warning>'.$lockFile.' is present but ignored as the "lock" config option is disabled.</warning>');
+            }
+
+            $locker = new Package\Locker($io, new JsonFile($config->get('lock') ? $lockFile : Platform::getDevNull(), null, $io), $im, file_get_contents($composerFile), $process);
+            $composer->setLocker($locker);
+        } elseif ($composer instanceof Composer) {
+            $locker = new Package\Locker($io, new JsonFile(Platform::getDevNull(), null, $io), $im, JsonFile::encode($localConfig), $process);
+            $composer->setLocker($locker);
+        }
+
         if ($composer instanceof Composer) {
             $globalComposer = null;
             if (realpath($config->get('home')) !== $cwd) {
@@ -416,17 +430,6 @@ class Factory
             $pm->loadInstalledPlugins();
         }
 
-        // init locker if possible
-        if ($composer instanceof Composer && isset($composerFile)) {
-            $lockFile = self::getLockFile($composerFile);
-            if (!$config->get('lock') && file_exists($lockFile)) {
-                $io->writeError('<warning>'.$lockFile.' is present but ignored as the "lock" config option is disabled.</warning>');
-            }
-
-            $locker = new Package\Locker($io, new JsonFile($config->get('lock') ? $lockFile : Platform::getDevNull(), null, $io), $im, file_get_contents($composerFile), $process);
-            $composer->setLocker($locker);
-        }
-
         if ($fullLoad) {
             $initEvent = new Event(PluginEvents::INIT);
             $composer->getEventDispatcher()->dispatch($initEvent->getName(), $initEvent);
