@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 
 /*
  * This file is part of Composer.
@@ -14,8 +14,10 @@ namespace Composer\Installer;
 
 use Composer\Composer;
 use Composer\IO\IOInterface;
+use Composer\PartialComposer;
 use Composer\Repository\InstalledRepositoryInterface;
 use Composer\Package\PackageInterface;
+use Composer\Plugin\PluginManager;
 use Composer\Util\Filesystem;
 use Composer\Util\Platform;
 use React\Promise\PromiseInterface;
@@ -28,13 +30,7 @@ use React\Promise\PromiseInterface;
  */
 class PluginInstaller extends LibraryInstaller
 {
-    /**
-     * Initializes Plugin installer.
-     *
-     * @param IOInterface $io
-     * @param Composer    $composer
-     */
-    public function __construct(IOInterface $io, Composer $composer, Filesystem $fs = null, BinaryInstaller $binaryInstaller = null)
+    public function __construct(IOInterface $io, PartialComposer $composer, Filesystem $fs = null, BinaryInstaller $binaryInstaller = null)
     {
         parent::__construct($io, $composer, 'composer-plugin', $fs, $binaryInstaller);
     }
@@ -42,11 +38,24 @@ class PluginInstaller extends LibraryInstaller
     /**
      * @inheritDoc
      */
-    public function supports($packageType)
+    public function supports(string $packageType)
     {
         return $packageType === 'composer-plugin' || $packageType === 'composer-installer';
     }
 
+    /**
+     * @inheritDoc
+     */
+    public function prepare($type, PackageInterface $package, PackageInterface $prevPackage = null)
+    {
+        // fail install process early if it going to fail due to a plugin not being allowed
+        if ($type === 'install' || $type === 'update') {
+            $this->getPluginManager()->isPluginAllowed($package->getName(), false);
+        }
+
+        return parent::prepare($type, $package, $prevPackage);
+    }
+
     /**
      * @inheritDoc
      */
@@ -67,18 +76,15 @@ class PluginInstaller extends LibraryInstaller
     {
         $promise = parent::install($repo, $package);
         if (!$promise instanceof PromiseInterface) {
-            $promise = \React\Promise\resolve();
+            $promise = \React\Promise\resolve(null);
         }
 
-        $pluginManager = $this->composer->getPluginManager();
-        $self = $this;
-
-        return $promise->then(function () use ($self, $pluginManager, $package, $repo) {
+        return $promise->then(function () use ($package, $repo): void {
             try {
                 Platform::workaroundFilesystemIssues();
-                $pluginManager->registerPackage($package, true);
+                $this->getPluginManager()->registerPackage($package, true);
             } catch (\Exception $e) {
-                $self->rollbackInstall($e, $repo, $package);
+                $this->rollbackInstall($e, $repo, $package);
             }
         });
     }
@@ -90,40 +96,39 @@ class PluginInstaller extends LibraryInstaller
     {
         $promise = parent::update($repo, $initial, $target);
         if (!$promise instanceof PromiseInterface) {
-            $promise = \React\Promise\resolve();
+            $promise = \React\Promise\resolve(null);
         }
 
-        $pluginManager = $this->composer->getPluginManager();
-        $self = $this;
-
-        return $promise->then(function () use ($self, $pluginManager, $initial, $target, $repo) {
+        return $promise->then(function () use ($initial, $target, $repo): void {
             try {
                 Platform::workaroundFilesystemIssues();
-                $pluginManager->deactivatePackage($initial);
-                $pluginManager->registerPackage($target, true);
+                $this->getPluginManager()->deactivatePackage($initial);
+                $this->getPluginManager()->registerPackage($target, true);
             } catch (\Exception $e) {
-                $self->rollbackInstall($e, $repo, $target);
+                $this->rollbackInstall($e, $repo, $target);
             }
         });
     }
 
     public function uninstall(InstalledRepositoryInterface $repo, PackageInterface $package)
     {
-        $this->composer->getPluginManager()->uninstallPackage($package);
+        $this->getPluginManager()->uninstallPackage($package);
 
         return parent::uninstall($repo, $package);
     }
 
-    /**
-     * TODO v3 should make this private once we can drop PHP 5.3 support
-     * @private
-     *
-     * @return void
-     */
-    public function rollbackInstall(\Exception $e, InstalledRepositoryInterface $repo, PackageInterface $package)
+    private function rollbackInstall(\Exception $e, InstalledRepositoryInterface $repo, PackageInterface $package): void
     {
         $this->io->writeError('Plugin initialization failed ('.$e->getMessage().'), uninstalling plugin');
         parent::uninstall($repo, $package);
         throw $e;
     }
+
+    protected function getPluginManager(): PluginManager
+    {
+        assert($this->composer instanceof Composer, new \LogicException(self::class.' should be initialized with a fully loaded Composer instance.'));
+        $pluginManager = $this->composer->getPluginManager();
+
+        return $pluginManager;
+    }
 }
