@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 
 /*
  * This file is part of Composer.
@@ -28,9 +28,9 @@ class FossilDriverTest extends TestCase
      */
     protected $config;
 
-    public function setUp()
+    public function setUp(): void
     {
-        $this->home = $this->getUniqueTmpDirectory();
+        $this->home = self::getUniqueTmpDirectory();
         $this->config = new Config();
         $this->config->merge(array(
             'config' => array(
@@ -39,13 +39,14 @@ class FossilDriverTest extends TestCase
         ));
     }
 
-    public function tearDown()
+    protected function tearDown(): void
     {
+        parent::tearDown();
         $fs = new Filesystem();
         $fs->removeDirectory($this->home);
     }
 
-    public static function supportProvider()
+    public static function supportProvider(): array
     {
         return array(
             array('http://fossil.kd2.org/kd2fw/', true),
@@ -60,7 +61,7 @@ class FossilDriverTest extends TestCase
      * @param string $url
      * @param bool   $assertion
      */
-    public function testSupport($url, $assertion)
+    public function testSupport(string $url, bool $assertion): void
     {
         $config = new Config();
         $result = FossilDriver::supports($this->getMockBuilder('Composer\IO\IOInterface')->getMock(), $config, $url);
