@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 
 /*
  * This file is part of Composer.
@@ -32,15 +32,16 @@ class FilesystemTest extends TestCase
      */
     private $testFile;
 
-    public function setUp()
+    public function setUp(): void
     {
         $this->fs = new Filesystem;
-        $this->workingDir = $this->getUniqueTmpDirectory();
-        $this->testFile = $this->getUniqueTmpDirectory() . '/composer_test_file';
+        $this->workingDir = self::getUniqueTmpDirectory();
+        $this->testFile = self::getUniqueTmpDirectory() . '/composer_test_file';
     }
 
-    public function tearDown()
+    protected function tearDown(): void
     {
+        parent::tearDown();
         if (is_dir($this->workingDir)) {
             $this->fs->removeDirectory($this->workingDir);
         }
@@ -58,13 +59,13 @@ class FilesystemTest extends TestCase
      * @param string $expected
      * @param bool   $static
      */
-    public function testFindShortestPathCode($a, $b, $directory, $expected, $static = false)
+    public function testFindShortestPathCode(string $a, string $b, bool $directory, string $expected, bool $static = false): void
     {
         $fs = new Filesystem;
         $this->assertEquals($expected, $fs->findShortestPathCode($a, $b, $directory, $static));
     }
 
-    public function providePathCouplesAsCode()
+    public function providePathCouplesAsCode(): array
     {
         return array(
             array('/foo/bar', '/foo/bar', false, "__FILE__"),
@@ -73,19 +74,19 @@ class FilesystemTest extends TestCase
             array('/foo/bin/run', '/bar/bin/run', false, "'/bar/bin/run'"),
             array('c:/bin/run', 'c:/vendor/acme/bin/run', false, "dirname(__DIR__).'/vendor/acme/bin/run'"),
             array('c:\\bin\\run', 'c:/vendor/acme/bin/run', false, "dirname(__DIR__).'/vendor/acme/bin/run'"),
-            array('c:/bin/run', 'd:/vendor/acme/bin/run', false, "'d:/vendor/acme/bin/run'"),
-            array('c:\\bin\\run', 'd:/vendor/acme/bin/run', false, "'d:/vendor/acme/bin/run'"),
+            array('c:/bin/run', 'D:/vendor/acme/bin/run', false, "'D:/vendor/acme/bin/run'"),
+            array('c:\\bin\\run', 'd:/vendor/acme/bin/run', false, "'D:/vendor/acme/bin/run'"),
             array('/foo/bar', '/foo/bar', true, "__DIR__"),
             array('/foo/bar/', '/foo/bar', true, "__DIR__"),
             array('/foo/bar', '/foo/baz', true, "dirname(__DIR__).'/baz'"),
             array('/foo/bin/run', '/foo/vendor/acme/bin/run', true, "dirname(dirname(__DIR__)).'/vendor/acme/bin/run'"),
             array('/foo/bin/run', '/bar/bin/run', true, "'/bar/bin/run'"),
             array('/bin/run', '/bin/run', true, "__DIR__"),
-            array('c:/bin/run', 'c:\\bin/run', true, "__DIR__"),
+            array('c:/bin/run', 'C:\\bin/run', true, "__DIR__"),
             array('c:/bin/run', 'c:/vendor/acme/bin/run', true, "dirname(dirname(__DIR__)).'/vendor/acme/bin/run'"),
             array('c:\\bin\\run', 'c:/vendor/acme/bin/run', true, "dirname(dirname(__DIR__)).'/vendor/acme/bin/run'"),
-            array('c:/bin/run', 'd:/vendor/acme/bin/run', true, "'d:/vendor/acme/bin/run'"),
-            array('c:\\bin\\run', 'd:/vendor/acme/bin/run', true, "'d:/vendor/acme/bin/run'"),
+            array('c:/bin/run', 'd:/vendor/acme/bin/run', true, "'D:/vendor/acme/bin/run'"),
+            array('c:\\bin\\run', 'd:/vendor/acme/bin/run', true, "'D:/vendor/acme/bin/run'"),
             array('C:/Temp/test', 'C:\Temp', true, "dirname(__DIR__)"),
             array('C:/Temp', 'C:\Temp\test', true, "__DIR__ . '/test'"),
             array('/tmp/test', '/tmp', true, "dirname(__DIR__)"),
@@ -95,7 +96,7 @@ class FilesystemTest extends TestCase
             array('/tmp/test/../vendor', '/tmp/test', true, "dirname(__DIR__).'/test'"),
             array('/tmp/test/.././vendor', '/tmp/test', true, "dirname(__DIR__).'/test'"),
             array('C:/Temp', 'c:\Temp\..\..\test', true, "dirname(__DIR__).'/test'"),
-            array('C:/Temp/../..', 'd:\Temp\..\..\test', true, "'d:/test'"),
+            array('C:/Temp/../..', 'd:\Temp\..\..\test', true, "'D:/test'"),
             array('/foo/bar', '/foo/bar_vendor', true, "dirname(__DIR__).'/bar_vendor'"),
             array('/foo/bar_vendor', '/foo/bar', true, "dirname(__DIR__).'/bar'"),
             array('/foo/bar_vendor', '/foo/bar/src', true, "dirname(__DIR__).'/bar/src'"),
@@ -105,7 +106,7 @@ class FilesystemTest extends TestCase
             array('/tmp/test/../vendor', '/tmp/test', true, "__DIR__ . '/..'.'/test'", true),
             array('/tmp/test/.././vendor', '/tmp/test', true, "__DIR__ . '/..'.'/test'", true),
             array('C:/Temp', 'c:\Temp\..\..\test', true, "__DIR__ . '/..'.'/test'", true),
-            array('C:/Temp/../..', 'd:\Temp\..\..\test', true, "'d:/test'", true),
+            array('C:/Temp/../..', 'd:\Temp\..\..\test', true, "'D:/test'", true),
             array('/foo/bar', '/foo/bar_vendor', true, "__DIR__ . '/..'.'/bar_vendor'", true),
             array('/foo/bar_vendor', '/foo/bar', true, "__DIR__ . '/..'.'/bar'", true),
             array('/foo/bar_vendor', '/foo/bar/src', true, "__DIR__ . '/..'.'/bar/src'", true),
@@ -121,13 +122,13 @@ class FilesystemTest extends TestCase
      * @param string $expected
      * @param bool   $directory
      */
-    public function testFindShortestPath($a, $b, $expected, $directory = false)
+    public function testFindShortestPath(string $a, string $b, string $expected, bool $directory = false): void
     {
         $fs = new Filesystem;
         $this->assertEquals($expected, $fs->findShortestPath($a, $b, $directory));
     }
 
-    public function providePathCouples()
+    public function providePathCouples(): array
     {
         return array(
             array('/foo/bar', '/foo/bar', "./bar"),
@@ -140,11 +141,11 @@ class FilesystemTest extends TestCase
             array('/foo/bin/run', '/foo/vendor/acme/bin/run', "../vendor/acme/bin/run"),
             array('/foo/bin/run', '/bar/bin/run', "/bar/bin/run"),
             array('/foo/bin/run', '/bar/bin/run', "/bar/bin/run", true),
-            array('c:/foo/bin/run', 'd:/bar/bin/run', "d:/bar/bin/run", true),
+            array('c:/foo/bin/run', 'd:/bar/bin/run', "D:/bar/bin/run", true),
             array('c:/bin/run', 'c:/vendor/acme/bin/run', "../vendor/acme/bin/run"),
             array('c:\\bin\\run', 'c:/vendor/acme/bin/run', "../vendor/acme/bin/run"),
-            array('c:/bin/run', 'd:/vendor/acme/bin/run', "d:/vendor/acme/bin/run"),
-            array('c:\\bin\\run', 'd:/vendor/acme/bin/run', "d:/vendor/acme/bin/run"),
+            array('c:/bin/run', 'd:/vendor/acme/bin/run', "D:/vendor/acme/bin/run"),
+            array('c:\\bin\\run', 'd:/vendor/acme/bin/run', "D:/vendor/acme/bin/run"),
             array('C:/Temp/test', 'C:\Temp', "./"),
             array('/tmp/test', '/tmp', "./"),
             array('C:/Temp/test/sub', 'C:\Temp', "../"),
@@ -159,7 +160,7 @@ class FilesystemTest extends TestCase
             array('/tmp/test/.././vendor', '/tmp/test', '../test', true),
             array('C:/Temp', 'c:\Temp\..\..\test', "../test", true),
             array('C:/Temp/../..', 'c:\Temp\..\..\test', "./test", true),
-            array('C:/Temp/../..', 'D:\Temp\..\..\test', "d:/test", true),
+            array('C:/Temp/../..', 'D:\Temp\..\..\test', "D:/test", true),
             array('/tmp', '/tmp/../../test', '/test', true),
             array('/foo/bar', '/foo/bar_vendor', '../bar_vendor', true),
             array('/foo/bar_vendor', '/foo/bar', '../bar', true),
@@ -172,7 +173,7 @@ class FilesystemTest extends TestCase
     /**
      * @group GH-1339
      */
-    public function testRemoveDirectoryPhp()
+    public function testRemoveDirectoryPhp(): void
     {
         @mkdir($this->workingDir . "/level1/level2", 0777, true);
         file_put_contents($this->workingDir . "/level1/level2/hello.txt", "hello world");
@@ -182,7 +183,7 @@ class FilesystemTest extends TestCase
         $this->assertFileDoesNotExist($this->workingDir . "/level1/level2/hello.txt");
     }
 
-    public function testFileSize()
+    public function testFileSize(): void
     {
         file_put_contents($this->testFile, 'Hello');
 
@@ -190,7 +191,7 @@ class FilesystemTest extends TestCase
         $this->assertGreaterThanOrEqual(5, $fs->size($this->testFile));
     }
 
-    public function testDirectorySize()
+    public function testDirectorySize(): void
     {
         @mkdir($this->workingDir, 0777, true);
         file_put_contents($this->workingDir."/file1.txt", 'Hello');
@@ -206,34 +207,34 @@ class FilesystemTest extends TestCase
      * @param string $expected
      * @param string $actual
      */
-    public function testNormalizePath($expected, $actual)
+    public function testNormalizePath(string $expected, string $actual): void
     {
         $fs = new Filesystem;
         $this->assertEquals($expected, $fs->normalizePath($actual));
     }
 
-    public function provideNormalizedPaths()
+    public function provideNormalizedPaths(): array
     {
         return array(
             array('../foo', '../foo'),
-            array('c:/foo/bar', 'c:/foo//bar'),
+            array('C:/foo/bar', 'c:/foo//bar'),
             array('C:/foo/bar', 'C:/foo/./bar'),
             array('C:/foo/bar', 'C://foo//bar'),
             array('C:/foo/bar', 'C:///foo//bar'),
             array('C:/bar', 'C:/foo/../bar'),
             array('/bar', '/foo/../bar/'),
-            array('phar://c:/Foo', 'phar://c:/Foo/Bar/..'),
-            array('phar://c:/Foo', 'phar://c:///Foo/Bar/..'),
-            array('phar://c:/', 'phar://c:/Foo/Bar/../../../..'),
+            array('phar://C:/Foo', 'phar://c:/Foo/Bar/..'),
+            array('phar://C:/Foo', 'phar://c:///Foo/Bar/..'),
+            array('phar://C:/', 'phar://c:/Foo/Bar/../../../..'),
             array('/', '/Foo/Bar/../../../..'),
             array('/', '/'),
             array('/', '//'),
             array('/', '///'),
             array('/Foo', '///Foo'),
-            array('c:/', 'c:\\'),
+            array('C:/', 'c:\\'),
             array('../src', 'Foo/Bar/../../../src'),
-            array('c:../b', 'c:.\\..\\a\\..\\b'),
-            array('phar://c:../Foo', 'phar://c:../Foo'),
+            array('C:../b', 'c:.\\..\\a\\..\\b'),
+            array('phar://C:../Foo', 'phar://c:../Foo'),
             array('//foo/bar', '\\\\foo\\bar'),
         );
     }
@@ -242,7 +243,7 @@ class FilesystemTest extends TestCase
      * @link https://github.com/composer/composer/issues/3157
      * @requires function symlink
      */
-    public function testUnlinkSymlinkedDirectory()
+    public function testUnlinkSymlinkedDirectory(): void
     {
         $basepath = $this->workingDir;
         $symlinked = $basepath . "/linked";
@@ -269,7 +270,7 @@ class FilesystemTest extends TestCase
      * @link https://github.com/composer/composer/issues/3144
      * @requires function symlink
      */
-    public function testRemoveSymlinkedDirectoryWithTrailingSlash()
+    public function testRemoveSymlinkedDirectoryWithTrailingSlash(): void
     {
         @mkdir($this->workingDir . "/real", 0777, true);
         touch($this->workingDir . "/real/FILE");
@@ -298,7 +299,7 @@ class FilesystemTest extends TestCase
         $this->assertFileDoesNotExist($symlinked);
     }
 
-    public function testJunctions()
+    public function testJunctions(): void
     {
         @mkdir($this->workingDir . '/real/nesting/testing', 0777, true);
         $fs = new Filesystem();
@@ -307,7 +308,8 @@ class FilesystemTest extends TestCase
         if (!defined('PHP_WINDOWS_VERSION_BUILD')) {
             $this->assertFalse($fs->isJunction($this->workingDir));
             $this->assertFalse($fs->removeJunction($this->workingDir));
-            $this->setExpectedException('LogicException', 'not available on non-Windows platform');
+            self::expectException('LogicException');
+            self::expectExceptionMessage('not available on non-Windows platform');
         }
 
         $target = $this->workingDir . '/real/../real/nesting';
@@ -327,7 +329,7 @@ class FilesystemTest extends TestCase
         $this->assertFalse(is_dir($junction), $junction . ' is not a directory');
     }
 
-    public function testCopy()
+    public function testCopy(): void
     {
         @mkdir($this->workingDir . '/foo/bar', 0777, true);
         @mkdir($this->workingDir . '/foo/baz', 0777, true);
@@ -352,7 +354,7 @@ class FilesystemTest extends TestCase
         $this->assertTrue(is_file($this->workingDir . '/testfile.file'));
     }
 
-    public function testCopyThenRemove()
+    public function testCopyThenRemove(): void
     {
         @mkdir($this->workingDir . '/foo/bar', 0777, true);
         @mkdir($this->workingDir . '/foo/baz', 0777, true);
