@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 
 /*
  * This file is part of Composer.
@@ -17,7 +17,7 @@ use Composer\Test\TestCase;
 
 class ProxyHelperTest extends TestCase
 {
-    protected function setUp()
+    protected function setUp(): void
     {
         unset(
             $_SERVER['HTTP_PROXY'],
@@ -30,8 +30,9 @@ class ProxyHelperTest extends TestCase
         );
     }
 
-    protected function tearDown()
+    protected function tearDown(): void
     {
+        parent::tearDown();
         unset(
             $_SERVER['HTTP_PROXY'],
             $_SERVER['http_proxy'],
@@ -48,15 +49,15 @@ class ProxyHelperTest extends TestCase
      *
      * @param string $url
      */
-    public function testThrowsOnMalformedUrl($url)
+    public function testThrowsOnMalformedUrl(string $url): void
     {
         $_SERVER['http_proxy'] = $url;
 
-        $this->setExpectedException('RuntimeException');
+        self::expectException('RuntimeException');
         ProxyHelper::getProxyData();
     }
 
-    public function dataMalformed()
+    public function dataMalformed(): array
     {
         return array(
             'no-host' => array('localhost'),
@@ -70,7 +71,7 @@ class ProxyHelperTest extends TestCase
      * @param string $url
      * @param string $expected
      */
-    public function testUrlFormatting($url, $expected)
+    public function testUrlFormatting(string $url, string $expected): void
     {
         $_SERVER['http_proxy'] = $url;
 
@@ -78,7 +79,7 @@ class ProxyHelperTest extends TestCase
         $this->assertSame($expected, $httpProxy);
     }
 
-    public function dataFormatting()
+    public function dataFormatting(): array
     {
         // url, expected
         return array(
@@ -95,7 +96,7 @@ class ProxyHelperTest extends TestCase
      * @param string               $expected
      * @param int                  $index
      */
-    public function testLowercaseOverridesUppercase(array $server, $expected, $index)
+    public function testLowercaseOverridesUppercase(array $server, string $expected, int $index): void
     {
         $_SERVER = array_merge($_SERVER, $server);
 
@@ -103,7 +104,7 @@ class ProxyHelperTest extends TestCase
         $this->assertSame($expected, $list[$index]);
     }
 
-    public function dataCaseOverrides()
+    public function dataCaseOverrides(): array
     {
         // server, expected, list index
         return array(
@@ -120,7 +121,7 @@ class ProxyHelperTest extends TestCase
      * @param string               $expected
      * @param int                  $index
      */
-    public function testCGIUpperCaseOverridesHttp(array $server, $expected, $index)
+    public function testCGIUpperCaseOverridesHttp(array $server, string $expected, int $index): void
     {
         $_SERVER = array_merge($_SERVER, $server);
 
@@ -128,7 +129,7 @@ class ProxyHelperTest extends TestCase
         $this->assertSame($expected, $list[$index]);
     }
 
-    public function dataCGIOverrides()
+    public function dataCGIOverrides(): array
     {
         // server, expected, list index
         return array(
@@ -137,7 +138,7 @@ class ProxyHelperTest extends TestCase
         );
     }
 
-    public function testNoHttpsProxyUsesHttpProxy()
+    public function testNoHttpsProxyUsesHttpProxy(): void
     {
         $_SERVER['http_proxy'] = 'http://http.com';
 
@@ -145,7 +146,7 @@ class ProxyHelperTest extends TestCase
         $this->assertSame('http://http.com:80', $httpsProxy);
     }
 
-    public function testNoHttpProxyDoesNotUseHttpsProxy()
+    public function testNoHttpProxyDoesNotUseHttpsProxy(): void
     {
         $_SERVER['https_proxy'] = 'http://https.com';
 
@@ -161,12 +162,12 @@ class ProxyHelperTest extends TestCase
      *
      * @phpstan-param array{http: array{proxy: string, header?: string}} $expected
      */
-    public function testGetContextOptions($url, $expected)
+    public function testGetContextOptions(string $url, array $expected): void
     {
         $this->assertEquals($expected, ProxyHelper::getContextOptions($url));
     }
 
-    public function dataContextOptions()
+    public function dataContextOptions(): array
     {
         // url, expected
         return array(
@@ -189,7 +190,7 @@ class ProxyHelperTest extends TestCase
      * @param string  $requestUrl
      * @param mixed[] $expected
      */
-    public function testSetRequestFullUri($requestUrl, $expected)
+    public function testSetRequestFullUri(string $requestUrl, array $expected): void
     {
         $options = array();
         ProxyHelper::setRequestFullUri($requestUrl, $options);
@@ -197,7 +198,7 @@ class ProxyHelperTest extends TestCase
         $this->assertEquals($expected, $options);
     }
 
-    public function dataRequestFullUri()
+    public function dataRequestFullUri(): array
     {
         $options = array('http' => array('request_fulluri' => true));
 
